C$Procedure EXTAPP ( Time of bracketed extreme approach )

      SUBROUTINE EXTAPP ( TARG, CORR, OBS, REF, BEG, END,
     .                    TIME, STATE )

C$ Abstract
C
C     Return the time and distance of orbital extreme approach 
C     (peri- or apo- apsis) of an observer body to a specified
C     target body, within a specified time interval.
C
C$ Copyright
C
C     Copyright (1996), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SPK
C
C$ Keywords
C
C     EPHEMERIS
C     SEARCH
C     TIME
C     EXTREME
C
C$ Declarations


      INTEGER               TARG
      INTEGER               OBS
      CHARACTER*(*)         REF
      CHARACTER*(*)         CORR
      DOUBLE PRECISION      BEG
      DOUBLE PRECISION      END
      DOUBLE PRECISION      TIME
      DOUBLE PRECISION      STATE ( 6 )



C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     TARG       I   NAIF integer code of the target body.
C     CORR       I   Aberration correction.
C     OBS        I   NAIF integer code of the observer body.
C     REF        I   Reference frame for state.
C     BEG,
C     END        I   Begin and end times of search interval.
C     TIME       O   Time of extremum.
C     STATE      O   State vector at time of extremum(center-to-center).
C
C$ Detailed_Input
C
C     TARG           NAIF integer code of the target body.
C
C     CORR           Aberration correction.  Possible values are:
C
C                       'NONE'      {none}
C                       'LT'        {light time only}
C                       'LT+S'      {light time plus stellar aberration}
C
C                    The observer-target vector whose norm is minimized
C                    at closest approach will be corrected as specified
C                    by this argument.
C
C     OBS            NAIF integer code of the observer body.
C
C     REF            The reference fram, J2000, J1950, etc., for the
C                    state calculations.
C
C     BEG,
C     END            Begin and end times of search interval.  Both
C                    times are specified as ephemeris seconds past
C                    J2000.  The closest approach of the observer
C                    body to the target body within the time interval
C
C                       [ BEG, END ]
C
C                    will be found.
C
C                    BEG and END should be chosen so that a unique local
C                    minimum of the observer-target distance exists
C                    within the time interval they delimit; this
C                    condition is not checked (hence, not enforced) by
C                    this routine.
C
C$ Detailed_Output
C
C     TIME           is the time of extreme approach (peri
C                    or apo-apsis) of the observer to the
C                    target, within the interval specified
C                    by BEG and END, where the observer-target
C                    distance is computed using the aberration
C                    correction specified by the input argument
C                    CORR.
C
C     STATE          the state vector between the TARGET and OBS, from
C                    OBS to TARGET, at the time of extremum given the
C                    requested corrections.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1)  If the radial velocity of the target with respect to the
C         observer does not have opposite signs at BEG and END
C         the error SPICE(NOT BRACKETED) is signaled.
C
C     2)  If END is less than or equal to BEG, the error
C         SPICE(BAD END POINTS) will be signaled.
C
C     3)  If BEG and END delimit a time interval in which there is are
C         multiple local minima of the observer-target distance, the
C         time at which any one of those minima is attained may be
C         picked as the time of closest approach.  For this reason,
C         BEG and END should be picked so as to uniquely determine
C         the minimum observer-target distance.
C
C     4)  If no SPK file containing ephemeris information for the
C         specified observer and target at the times of interest
C         has been loaded, the error will be diagnosed by routines
C         called by this routine.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     The routine attempts to find the zero of the radial velocity
C     between two bracketed times.  The radial velocities at the
C     two times must have different signs.
C
C$ Examples
C
C     None.
C
C$ Restrictions
C
C     Detection of the apoapsis is invalid for non-elliptic orbits.
C     Parabolic and hyperbolic orbits have minimum radial values, but
C     not maximum radial values.  Okay, they do have maximum radii but
C     at infinity and it takes forever to get there, so they don't
C     count.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     E.D. Wright    (JPL)
C     N.J. Bachman   (JPL)
C
C$ Version
C
C-    SPICELIB Version 2.7.0, 21-MAY-2002 (EDW)
C
C       Correct error messages to match NAIF conventions.
C      
C-    SPICELIB Version 2.6, 20-JAN-1999  (EDW)
C
C        The reference frame now passes from calling routine.
C
C-    SPICELIB Version 2.5, 20-FEB-1998  (EDW)
C
C        Replace references to RADVEL with RADSPD.
C
C-    Beta Version 2.2.1, 20-APR-1997 (EDW)
C
C        Expanded comment documentation, reset MXLOOP to 100.
C
C-    Beta Version 2.2.0, 20-APR-1997 (EDW)
C
C        Redesigned algorithm to find time of either min or max radial
C        distances.  In both cases radial velocity is zero.  The local
C        maximum of a function is the local minimum of the negative of
C        that function.  Corrected error in the logic defining when to
C        end a search.  Routine rename EXTAPP.
C
C-    Beta Version 2.0.0, 06-FEB-1995 (NJB)
C
C        Completely re-written to fix a bug and simplify the code.
C        The new routine works by finding a root of the radial speed
C        of the target, rather than by minimizing distance.
C
C-    Beta Version 1.1.0, 20-SEP-1993 (NJB)
C
C        RETURN statement added to error handling code in
C        endpoint check.
C
C-    Beta Version 1.0.0, 24-APR-1991 (NJB)
C
C-&
C


C
C     SPICELIB functions
C

      LOGICAL               RETURN
      LOGICAL               OPSGND

C
C     Local parameters
C
      INTEGER               MXLOOP
      PARAMETER           ( MXLOOP = 100 )

C
C     Local variables
C

      DOUBLE PRECISION      LEFT
      DOUBLE PRECISION      LT
      DOUBLE PRECISION      MIDDLE
      DOUBLE PRECISION      RIGHT
      DOUBLE PRECISION      VRADB
      DOUBLE PRECISION      VRADE
      DOUBLE PRECISION      VRADM
      DOUBLE PRECISION      RADSPD

C
C     A digitally realistic value to indicate convergence.  Note that
C     using a zero as an indicator is not realistic and probably
C     impossible to attain.
C
      DOUBLE PRECISION      CONMIN
      PARAMETER           ( CONMIN = 1D-15 )

      INTEGER               NLOOP



C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'EXTAPP' )
      END IF


C
C     Endpoints must be in order and must be distinct.
C

      IF ( END .LE. BEG ) THEN

         CALL SETMSG ( 'BEG is #; END is #'      )
         CALL ERRDP  ( '#',  BEG                 )
         CALL ERRDP  ( '#',  END                 )
         CALL SIGERR ( 'ORBNUM(BADENDPOINTS)'    )
         CALL CHKOUT ( 'EXTAPP'                  )
         RETURN

      END IF

C
C     Now we need to initialize the radial velocities at both endpoints
C     and prepare working variables to find the extremum.  The
C     conditions Vrad(BEG) > 0 and Vrad(END) < 0 should hold for a
C     bracketed apoapsis and Vrad(BEG) < 0 and Vrad(END) > 0 should
C     hold for a bracketed periapsis.
C
C

      VRADB = RADSPD( TARG, BEG, REF, CORR, OBS )
      VRADE = RADSPD( TARG, END, REF, CORR, OBS )

C
C     Check that the radial velocities at the endpoints have opposite
C     signs.
C
      IF ( .NOT. OPSGND ( VRADB, VRADE ) ) THEN

         CALL SETMSG ( 'Lead bracket radial velocity at #1 is #2'
     .              // 'Trail bracket radial velocity at #3 is #4' )
         CALL ERRDP  ( '#1',  BEG                                  )
         CALL ERRDP  ( '#2',  VRADB                                )
         CALL ERRDP  ( '#3',  END                                  )
         CALL ERRDP  ( '#4',  VRADE                                )
         CALL SIGERR ( 'ORBNUM(INTERVALNOTBRACKETED)'              )
         CALL CHKOUT ( 'EXTAPP'                                    )
         RETURN

      END IF


C
C     If we got this far, the root is bracketed.  Find the root by
C     bisection.  Just to be safe, use an iteration limit.
C
      NLOOP  = 0
      LEFT   = BEG
      RIGHT  = END
      MIDDLE = 0.5D0 * ( BEG + END )
      VRADM  = RADSPD( TARG, MIDDLE, REF, CORR, OBS )


C
C     We stop if any of the following conditions occur:
C
C        - We can't distinguish the midpoint of the search interval
C          from an endpoint.
C
C        - We find an approximately exact root, to roundoff.  An
C          IMPORTANT NOTE:  It is not possible to exactly determine the
C          root as computers use finite arithmetic, but we can get
C          as close as we want (well, as close as we want respecting the
C          word size of our machine).
C
C        - We've looped too many times. Oops!
C
C

      DO WHILE (       ( MIDDLE       .GT. LEFT    )
     .           .AND. ( MIDDLE       .LT. RIGHT   )
     .           .AND. ( DABS(VRADM)  .GT. CONMIN  )
     .           .AND. ( NLOOP        .LT. MXLOOP  ) )



C
C        We shrink the search interval by setting the right or left
C        endpoint equal to the midpoint.  We decide which endpoint to
C        reset depending on the sign of the mid point velocity
C        and the sign of the beginning of interval velocity.
C

         IF ( VRADM .GT. 0 .AND. VRADB .LT. 0.D0 ) THEN

            RIGHT  =  MIDDLE
            VRADE  =  VRADM

         ELSE IF ( VRADM .LT. 0 .AND. VRADB .LT. 0.D0 ) THEN

            LEFT   =  MIDDLE
            VRADB  =  VRADM

         ELSE IF ( VRADM .GT. 0 .AND. VRADB .GT. 0.D0 ) THEN

            LEFT   =  MIDDLE
            VRADB  =  VRADM


         ELSE IF ( VRADM .LT. 0 .AND. VRADB .GT. 0.D0 ) THEN

            RIGHT  =  MIDDLE
            VRADE  =  VRADM

         END IF



C
C        Find the new midpoint.
C
         MIDDLE = 0.5D0 * ( LEFT + RIGHT )
         VRADM  = RADSPD( TARG, MIDDLE, REF, CORR, OBS )
         NLOOP  = NLOOP  + 1

      END DO


C
C     Fill the variables from the argument list with their appropriate
C     vales.  Make a last call to SPKEZ to get the state at the root
C     time.
C
      TIME =  MIDDLE

      CALL SPKEZ ( TARG, TIME, REF, CORR, OBS, STATE, LT )


C
C     Checkout and return to the main routine.
C
      CALL CHKOUT ( 'EXTAPP' )


      RETURN
      END

