C$Procedure SPKWSF ( Write SPK scratch information file )

      SUBROUTINE SPKWSF ( HANDLE, SCUNIT  )

C$ Abstract
C
C     Subroutine to create a scratch information file which contains
C     body, body center and time interval information for an SPK file.
C     The file may be accessed at any time after its creation.
C
C$ Copyright
C
C     Copyright (1996), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     SPK scratch file
C
C$ Declarations


      INTEGER               HANDLE
      INTEGER               SCUNIT


C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C      HANDLE    I    Handle of the SPK file to be summarized.
C      SCUNIT    O    Unit number for the scratch file.  Returned to
C                     main routine.
C      BODY     O-S   Body found in segment, to scratch file.
C      CENTER   O-S   Orbit center (foci) for BODY, to scratch file.
C      BEGTIM   O-S   Start time J2000 for segment data for BODY, to
C                     scratch file.
C      ENDTIM   O-S   END time J2000 for segment data for BODY, to
C                     scratch file.
C
C$ Detailed_Input
C
C      HANDLE         The handle assigned to the SPK kernel when
C                     the kernel is loaded.
C
C$ Detailed_Output
C
C      SCUNIT         The I/O unit number assigned to the scratch file
C                     when the file is created.  This value is returned
C                     the calling routine.
C
C      BODY           NAIF ID for the body whose position and velocity
C                     is described in the SPK segment.  Value written to
C                     scratch file.
C
C      CENTER         NAIF ID for the primary of BODY.  Value written to
C                     scratch file.
C
C      BEGTIM         Earliest ephemeris time for position/velocity in
C                     the segment (i.e. the beginning time of the
C                     segment).  Value written to scratch file.
C
C      ENDTIM         Latest ephemeris time for position/velocity in
C                     the segment (i.e. the end time of the segment).
C                     Value written to scratch file.
C
C$ Parameters
C
C      None.
C
C$ Files
C
C      The routine creates a scratch file which exists for the duration
C      of the programs run.
C
C$ Exceptions
C
C      None.
C
C$ Particulars
C
C      None.
C
C$ Examples
C
C      Open an SPK kernel and pass its handle to SPKWSF which will
C      write a scratch file containing the body center, body, min time
C      and max time for every segment in the SPK file.
C
C      CALL SPKLEF ( '/kernel/spk/pff.bsp', HNDLE )
C      CALL SPKWSF( HNDLE, SCUNIT)
C
C      The scratch file may be accessed from any point in the program
C      once created.  The file is identified by the SCUNIT identifier.
C      When SCUNIT is closed the file will be deleted.
C
C$ Restrictions
C
C      This routine must be called with a correct handle after the
C      kernel is loaded.
C
C$ Author_and_Institution
C
C      Ed Wright JPL
C
C$ Literature_References
C
C      None.
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 6-Aug-1996 (EDW)
C
C-&


C$ Index_Entries
C
C     SPK scratch file
C
C-&
C


C
C     SPICELIB FUNCTIONS
C

      LOGICAL               FAILED
      LOGICAL               RETURN


C
C     LOCAL PARAMETERS
C
C     Set up mnemonic names for the target body and the center body.
C
      INTEGER               BODIDX
      PARAMETER           ( BODIDX = 1          )

      INTEGER               CENIDX
      PARAMETER           ( CENIDX = BODIDX + 1 )


C
C     Set up mnemonic names for the segment begin and end time indices.
C
      INTEGER               BTMIDX
      PARAMETER           ( BTMIDX = 1          )

      INTEGER               ETMIDX
      PARAMETER           ( ETMIDX = BTMIDX + 1 )


C
C     SPK VALUES FOR ND, NI, NDESC
C
      INTEGER               ND
      PARAMETER           ( ND = 2 )

      INTEGER               NI
      PARAMETER           ( NI = 6 )

      INTEGER               NDESC
      PARAMETER           ( NDESC =  ND + (NI+1)/2 )

C
C     MAXIMUM NUMBER OF SEGMENTS ALLOWED IN A FILE.
C
      INTEGER               LBCELL
      PARAMETER           ( LBCELL = -5 )

      INTEGER               MAXWIN
      PARAMETER           ( MAXWIN = 500 )


C
C     LOCAL VARIABLES
C
      DOUBLE PRECISION      DESCR  ( NDESC  )
      DOUBLE PRECISION      DSUM   ( ND     )
      DOUBLE PRECISION      BEGTIM
      DOUBLE PRECISION      ENDTIM
      DOUBLE PRECISION      WINDOW (LBCELL:MAXWIN)

      INTEGER               ISUM   ( NI     )
      INTEGER               BODY
      INTEGER               CENTER

      LOGICAL               FOUND
      LOGICAL               FIRST


C
C     Saved variables
C
      SAVE                  FIRST

C
C     Initial value of save variable.
C
      DATA  FIRST  / .TRUE. /


C
C     Standard SPICE error check and result handling.
C
      IF ( RETURN() ) THEN
         RETURN
      END IF



      CALL CHKIN ( 'SPKWSF' )

C
C     Get a free logical number for the scratch file, and open the
C     scratch file if this is the first time into the routine.
C
      IF ( FIRST ) THEN

         CALL GETLUN ( SCUNIT )

         OPEN ( UNIT = SCUNIT, STATUS = 'SCRATCH' )

         FIRST = .FALSE.

      END IF




C
C     Initialize the window.
C
      CALL SSIZED( MAXWIN, WINDOW )

C
C     Get the number of data segments in the kernel..
C
      CALL DAFBFS ( HANDLE )

      FOUND  = .TRUE.


      DO WHILE ( FOUND )

         CALL DAFFNA( FOUND )

C
C        Error return if there was a problem with DAFFNA.
C
         IF ( FAILED() ) THEN

            CALL CHKOUT ( 'SPKWSF' )
            RETURN

         END IF


C
C     We have a segment.  For each segment get the summary and the
C     relevant information.
C
         IF ( FOUND ) THEN

            CALL DAFGS  ( DESCR                     )
            CALL DAFUS  ( DESCR, ND, NI, DSUM, ISUM )

            IF ( FAILED() ) THEN
               CALL CHKOUT ( 'SPKWSF'  )
               RETURN
            END IF


C
C           Decode the information concerning the segment.
C
            BEGTIM = DSUM ( BTMIDX )
            ENDTIM = DSUM ( ETMIDX )
            BODY   = ISUM ( BODIDX )
            CENTER = ISUM ( CENIDX )

C
C           Write the body, body center, begin and end ET times to the
C           scratch file for later access.
C
            WRITE( SCUNIT, * )  BODY, CENTER, BEGTIM, ENDTIM

         END IF


      END DO


      CALL CHKOUT ( 'SPKWSF' )

      RETURN
      END
