C$Procedure CRSSEQ ( Time of bracketed equator crossing )

      SUBROUTINE CRSSEQ ( TARG, CORR, OBS, REF, BEG, END,
     .                    TIME, STATE )

C$ Abstract
C
C     Return the time and state of a body passing through a node
C     ascending or descending.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     SPK
C
C$ Keywords
C
C     EPHEMERIS
C     SEARCH
C     TIME
C     NODE
C
C$ Declarations

      IMPLICIT NONE

      INTEGER               TARG
      INTEGER               OBS
      CHARACTER*(*)         REF
      CHARACTER*(*)         CORR
      DOUBLE PRECISION      BEG
      DOUBLE PRECISION      END
      DOUBLE PRECISION      TIME
      DOUBLE PRECISION      STATE ( 6 )



C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     TARG       I   NAIF integer code of the target body.
C     CORR       I   Aberration correction.
C     OBS        I   NAIF integer code of the observer body.
C     REF        I   Reference frame for state.
C     BEG,
C     END        I   Begin and end times of search interval.
C     TIME       O   Time of extremum.
C     STATE      O   State vector at time of extremum(center-to-center).
C
C$ Detailed_Input
C
C     TARG           NAIF integer code of the target body.
C
C     CORR           Aberration correction.  Possible values are:
C
C                       'NONE'      {none}
C                       'LT'        {light time only}
C                       'LT+S'      {light time plus stellar aberration}
C
C     OBS            NAIF integer code of the observer body.
C
C     REF            The reference fram, J2000, J1950, etc., for the
C                    state calculations.
C
C     BEG,
C     END            Begin and end times of search interval.  Both
C                    times are specified as ephemeris seconds past
C                    J2000.  The closest approach of the observer
C                    body to the target body within the time interval
C
C                       [ BEG, END ]
C
C                    will be found.
C
C                    BEG and END should be chosen so that a unique
C                    Z=0 condition of the target position exists
C                    within the time interval they delimit; this
C                    condition is not checked (hence, not enforced) by
C                    this routine.
C
C$ Detailed_Output
C
C     TIME           is the time of the Z=0 crossing for the target,
C                    within the interval specified
C                    by BEG and END, where the observer-target
C                    distance is computed using the aberration
C                    correction specified by the input argument
C                    CORR.
C
C     STATE          the state vector between the TARGET and OBS, from
C                    OBS to TARGET, at the time of Z=0 given the
C                    requested corrections.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1)  If the Z component of the target's position with respect to
C         the observer does not have opposite signs at BEG and END
C         the error SPICE(NOT BRACKETED) is signaled.
C
C     2)  If END is less than or equal to BEG, the error
C         SPICE(BAD END POINTS) will be signaled.
C
C     3)  If BEG and END delimit a time interval in which
C         multiple Z=0 events occur, the time of any one of those events
C         may be picked as the time of event.  For this reason,
C         BEG and END should be picked so as to uniquely determine
C         the Z=0 event.
C
C     4)  If no SPK file containing ephemeris information for the
C         specified observer and target at the times of interest
C         has been loaded, the error will be diagnosed by routines
C         called by this routine.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     The routine attempts to find the zero of the Z component of
C     a position vector between two bracketed times.  The input times
C     ETB and ETF must map to Z components values with opposite signs.
C
C$ Examples
C
C     None.
C
C$ Restrictions
C
C     Detection of the node is invalid for non-inclined orbits.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     E.D. Wright    (JPL)
C
C$ Version
C
C-   Beta Version 1.1.0, 21-MAY-2002 (EDW)
C
C      Correct error messages to match NAIF conventions.
C      
C-   Beta Version 1.0.0, 19-FEB-1999 (EDW)
C
C       Based on EXTAPP.
C
C-&

C
C     SPICELIB functions
C

      LOGICAL               RETURN
      LOGICAL               OPSGND

C
C     Local parameters
C
      INTEGER               MXLOOP
      PARAMETER           ( MXLOOP = 100 )

C
C     Local variables
C

      DOUBLE PRECISION      LEFT
      DOUBLE PRECISION      LT
      DOUBLE PRECISION      MIDDLE
      DOUBLE PRECISION      RIGHT
      DOUBLE PRECISION      ZBEG
      DOUBLE PRECISION      ZEND
      DOUBLE PRECISION      ZMID


C
C     A digitally realistic value to indicate convergence.  Note that
C     using a zero as an indicator is not realistic and probably
C     impossible to attain.
C
      DOUBLE PRECISION      CONMIN
      PARAMETER           ( CONMIN = 1D-15 )

      INTEGER               NLOOP



C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'CRSSEQ' )
      END IF


C
C     Endpoints must be in order and must be distinct.
C

      IF ( END .LE. BEG ) THEN

         CALL SETMSG ( 'BEG is #; END is #'      )
         CALL ERRDP  ( '#',  BEG                 )
         CALL ERRDP  ( '#',  END                 )
         CALL SIGERR ( 'ORBNUM(BADENDPOINTS)'    )
         CALL CHKOUT ( 'CRSSEQ'                  )
         RETURN

      END IF

C
C     Now we need to initialize the Z components of the position vector
C     in the body fixed frame.  The conditions Z(BEG) > 0 and Z(END) < 0
C     or Z(BEG) < 0 and Z(END) > 0 will be true for a bracketed node
C     crossing.
C
C

      CALL SPKEZ ( TARG, BEG, REF, CORR, OBS, STATE, LT )
      ZBEG = STATE( 3 )

      CALL SPKEZ ( TARG, END, REF, CORR, OBS, STATE, LT )
      ZEND = STATE( 3 )

C
C     Check that the Z components at the endpoints have opposite
C     signs.
C
      IF ( .NOT. OPSGND ( ZBEG, ZEND ) ) THEN

         CALL SETMSG ( 'Lead bracket Z component at #1 is #2 '
     .              // 'Trail bracket Z component at #3 is #4')
         CALL ERRDP  ( '#1',  BEG                              )
         CALL ERRDP  ( '#2',  ZBEG                             )
         CALL ERRDP  ( '#3',  END                              )
         CALL ERRDP  ( '#4',  ZEND                             )
         CALL SIGERR ( 'ORBNUM(INTERVALNOTBRACKETED)'          )
         CALL CHKOUT ( 'CRSSEQ'                                )
         RETURN

      END IF


C
C     If we got this far, the root is bracketed.  Find the root by
C     bisection.  Just to be safe, use an iteration limit.
C
      NLOOP  = 0
      LEFT   = BEG
      RIGHT  = END
      MIDDLE = 0.5D0 * ( BEG + END )

      CALL SPKEZ ( TARG, MIDDLE, REF, CORR, OBS, STATE, LT )
      ZMID   = STATE( 3 )


C
C     We stop if any of the following conditions occur:
C
C        - We can't distinguish the midpoint of the search interval
C          from an endpoint.
C
C        - We find an approximately exact root, to roundoff.  An
C          IMPORTANT NOTE:  It is not possible to exactly determine the
C          root as computers use finite arithmetic, but we can get
C          as close as we want (well, as close as we want respecting the
C          word size of our machine).
C
C        - We've looped too many times. Oops!
C
C

      DO WHILE (       ( MIDDLE       .GT. LEFT    )
     .           .AND. ( MIDDLE       .LT. RIGHT   )
     .           .AND. ( DABS(ZMID)   .GT. CONMIN  )
     .           .AND. ( NLOOP        .LT. MXLOOP  ) )



C
C        We shrink the search interval by setting the right or left
C        endpoint equal to the midpoint.  We decide which endpoint to
C        reset depending on the sign of the mid point velocity
C        and the sign of the beginning of interval velocity.
C

         IF ( ZMID .GT. 0 .AND. ZBEG .LT. 0.D0 ) THEN

            RIGHT = MIDDLE
            ZEND  = ZMID

         ELSE IF ( ZMID .LT. 0 .AND. ZBEG .LT. 0.D0 ) THEN

            LEFT = MIDDLE
            ZBEG = ZMID

         ELSE IF ( ZMID .GT. 0 .AND. ZBEG .GT. 0.D0 ) THEN

            LEFT = MIDDLE
            ZBEG = ZMID


         ELSE IF ( ZMID .LT. 0 .AND. ZBEG .GT. 0.D0 ) THEN

            RIGHT = MIDDLE
            ZEND  = ZMID

         END IF



C
C        Find the new midpoint.
C
         MIDDLE = 0.5D0 * ( LEFT + RIGHT )

         CALL SPKEZ ( TARG, MIDDLE, REF, CORR, OBS, STATE, LT )
         ZMID   = STATE( 3 )
         NLOOP  = NLOOP  + 1

      END DO


C
C     Fill the variables from the argument list with their appropriate
C     values.  Make a last call to SPKEZ to get the state at the root
C     time.
C
      TIME =  MIDDLE

      CALL SPKEZ ( TARG, TIME, REF, CORR, OBS, STATE, LT )


C
C     Checkout and return to the main routine.
C
      CALL CHKOUT ( 'CRSSEQ' )


      RETURN
      END



