
C$Procedure CRSSND ( Time and state of node crossing )

      SUBROUTINE CRSSND ( TARGET, OBS , STRTTM, REF   , ETMIN, ETMAX,
     .                    CORR  , MU  , NODE  , EVNTTM, STATE, ALLGD )
 
C$ Abstract
C
C     Routine to return the time of crossing an ascending or descending
C     node.
C
C$ Copyright
C
C     Copyright (1999), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     NODES
C
C$ Declarations

      IMPLICIT NONE
 
      INTEGER                  TARGET
      INTEGER                  OBS
      DOUBLE PRECISION         STRTTM
      CHARACTER*(*)            REF
      DOUBLE PRECISION         ETMIN
      DOUBLE PRECISION         ETMAX
      CHARACTER*(*)            CORR
      DOUBLE PRECISION         MU
      INTEGER                  NODE
      DOUBLE PRECISION         EVNTTM
      LOGICAL                  FOUND

C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     TARGET     I   NAIF integer code of the target body.
C     OBS        I   NAIF integer code of the observer body.
C     STRTTM     I   Seach start time.
C     REF        I   Reference frame for state.
C     ETMIN      I   Minimum value for time bracket.
C     ETMAX      I   Maximum value for time bracket.
C     CORR       I   Aberration correction.
C     MU         I   Gravitational parameter for OBS body.
C     NODE       I   Flag to signal ascending or descending node search.
C     EVNTTM     0   Time of extremum.
C     STATE      O   State vector at time of extremum(center-to-center).
C     ALLGD      O   Boolean, true if node found, false otherwise.
C
C$ Detailed_Input
C
C     TARGET         NAIF integer code of the target body.
C
C     OBS            NAIF integer code of the observer body.
C    
C     STRTTM         Time at which search for node begins.
C
C     REF            The reference fram, J2000, J1950, etc., for the
C                    state calculations.
C
C     ETMIN          Minimum time bounds for search.  All time values
C                    calculated in this routine must be later than this
C                    time, otherwise ALLGD is set to false.
C
C     ETMAX          Maxmum time bounds for search.  All time values
C                    calculated in this routine must be earlier than
C                    this time, otherwise ALLGD is set to false.
C
C     CORR           Aberration correction.  Possible values are:
C
C                       'NONE'      {none}
C                       'LT'        {light time only}
C                       'LT+S'      {light time plus stellar aberration}
C
C     MU             Value of gravitation parameter mu for the observing
C                    body.
C
C     NODE           Flag which identifies whether the algorithm returns
C                    time-state of the ascending or descending node.   
C
C$ Detailed_Output
C
C     EVNTTM         Time of node crossing in ET J2000.
C
C     STATE          State vector for TARGET wrt OBS in REF with
C                    corrections CORR at EVNTTM
C
C     ALLGD          Boolean to indicate if a crossing was found.  If
C                    ALLGD = false, EVNTTM and STATE have no meaning.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     None.
C
C$ Exceptions
C
C     1)  The obit must have a non-zero inclination or an error
C         is flagged - no inclination, no nodes.
C
C     2)  The time of crossing must differ from both bracket times.
C
C$ Particulars
C
C     The routine determines the bracket times for the call to CRSSEQ
C     then uses that time to determine STATE and set ALLGD.  EVNTTM
C     exists in (STRTTM,ETMAX].
C
C     The algorithm to determine bracket times use is a simple
C     step-at-a-time algorithm with steps of 1/20 of the period.
C
C$ Examples
C
C    C
C    C Top of the Search/Scan loop.
C    C
C
C
C      DESCND = 1
C
C      DO WHILE( GO )
C
C         CALL CRSSND ( TARGET, OBS , ETB   , REF , ETS  , ETE,
C     .                 CORR  , MU  , DESCND, TIME, STATE, GO )
C
C         IF ( GO ) THEN
C
C         ...Do something if a crossing was found...
C
C         END IF
C
C
C      END DO
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     E.D. Wright      (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.1.0, 21-MAY-2002 (EDW)
C
C      Correct error messages to match NAIF conventions.
C      
C-    SPICELIB Version 1.0.0, 4-FEB-1999 (EDW)
C
C-&
C
C$ Index_Entries
C
C     ascending node
C     descending node
C
C-&


      DOUBLE PRECISION         ETB
      DOUBLE PRECISION         ETF
      DOUBLE PRECISION         LT
      DOUBLE PRECISION         PERIOD
      DOUBLE PRECISION         PI2
      DOUBLE PRECISION         TIME
      DOUBLE PRECISION         ELTS  ( 8 )
      DOUBLE PRECISION         STATE ( 6 )

      INTEGER                  I

      LOGICAL                  ALLGD
      LOGICAL                  FIRST

C
C     SPICELIB functions
C
      DOUBLE PRECISION         TWOPI
      LOGICAL                  RETURN
 

C
C     Save everything to preserve my sanity.
C
      SAVE

C
C     Set initialization flag.
C
      DATA                  FIRST  / .TRUE. /

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ('CRSSND' )
      END IF

      IF ( FIRST ) THEN
      
         PI2 = TWOPI()
      
      END IF
      

      ALLGD = .TRUE.
      FOUND = .TRUE.

C
C     Get our state at time.
C
      CALL SPKEZ  ( TARGET, STRTTM, REF, CORR, OBS, STATE, LT )
      CALL OSCELT ( STATE , STRTTM, MU, ELTS )

C
C     Check that the inclination is in fact non-zero.  If it is, stop.
C
      IF ( ELTS( 3 ) .EQ. 0.D0 ) THEN
         CALL SETMSG ( 'Zero inclination orbit.  No node crossings.' )
         CALL SIGERR ( 'ORBNUM(ZEROINCLINORBIT)' )
         CALL CHKOUT ( 'CRSSND'         )
         RETURN
      END IF


C     OSCELT returns the following
C
C                       RP      Perifocal distance.
C                       ECC     Eccentricity.
C                       INC     Inclination.
C                       LNODE   Longitude of the ascending node.
C                       ARGP    Argument of periapsis.
C                       M0      Mean anomaly at epoch.
C                       T0      Epoch.
C                       MU      Gravitational parameter.
C
C    So orbit period is obtained from
C
C                              RP    3
C                         ( ------- )
C                           1 - ECC        1/2
C     Period =  2 PI (   ---------------   )
C                             MU
C
      PERIOD = PI2 * DSQRT( (( ELTS(1) / (1.D0-ELTS(2)) )**3) / MU )


C
C     We want two times, ETB, ETF, such that
C     STATE(3)|      > 0.0
C             |(ETB)
C
C              and
C
C     STATE(3)|      < 0.0
C             |(ETF)
C     or
C     STATE(3)|      < 0.0
C             |(ETB)
C
C              and
C
C     STATE(3)|      > 0.0
C             |(ETF)
C     with ETB < ETF.
C

C
C     Get our state at time.
C
      I = 1

      CALL SPKEZ  ( TARGET, STRTTM, REF, CORR, OBS, STATE, LT )
      ETB   = STRTTM + DBLE(I)*0.05D0 * PERIOD
      ALLGD = ( ETB .GT. ETMIN .AND. ETB .LT. ETMAX )
            
      IF ( ALLGD ) THEN
         CALL SPKEZ  ( TARGET, ETB, REF, CORR, OBS, STATE, LT )
         I = I + 1
       END IF

C
C        Step forward in 1/20 increments of period to find a time when
C        NODE * STATE(3) > 0.0.
C
         DO WHILE ( NODE * STATE(3) .LT. 0.D0 .AND. ALLGD )

            ETB   = STRTTM + DBLE(I)*0.05D0 * PERIOD
            ALLGD = ( ETB .GT. ETMIN .AND. ETB .LT. ETMAX )
            
            IF ( ALLGD ) THEN
               CALL SPKEZ  ( TARGET, ETB, REF, CORR, OBS, STATE, LT )
               I = I + 1
            END IF

         END DO


      IF ( ALLGD ) THEN
      
         CALL SPKEZ  ( TARGET, ETB, REF, CORR, OBS, STATE, LT )

         I = 1
C
C        Step forward in 1/20 increments of period to find a time when
C        NODE * STATE(3) < 0.0.
C
         DO WHILE ( NODE * STATE(3) .GT. 0.D0 .AND. ALLGD )

            ETF   = ETB + DBLE(I)*0.05D0 * PERIOD
            ALLGD = ( ETF .GT. ETMIN .AND. ETF .LT. ETMAX )
            
            IF ( ALLGD ) THEN
               CALL SPKEZ  ( TARGET, ETF, REF, CORR, OBS, STATE, LT )
               I = I + 1
            END IF
         
         END DO
     
      END IF


      IF ( ALLGD ) THEN

         CALL CRSSEQ( TARGET, CORR, OBS, REF, ETB, ETF,
     .                TIME  , STATE )

         EVNTTM = TIME
      
         IF ( TIME .EQ. ETB .OR. TIME .EQ. ETF ) THEN
         
            CALL SETMSG ( 'Start time #1 equals first bracket'
     .                 // ' time #2 or last bracket time #3.')
            CALL ERRDP  ( '#1',  TIME                        )
            CALL ERRDP  ( '#2',  ETB                         )
            CALL ERRDP  ( '#3',  ETF                         )
            CALL SIGERR ( 'ORBNUM(INTERVALNOTBRACKETED)'     )
            CALL CHKOUT ( 'CRSSND'                           )

            RETURN
            
         END IF
      
      END IF
      
      CALL CHKOUT ('CRSSND' )
      
      RETURN
      END

