C$Procedure RADSPD (Return radial speed)

      FUNCTION RADSPD ( TARGET, ET, REF, CORR, OBS )

C$ Abstract
C
C     Procedure to determine the radial speed of a target w.r.t. a
C     observer at a specified J2000 ephemeris time.
C
C$ Copyright
C
C     Copyright (1996), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C    radial speed
C
C$ Declarations



      CHARACTER*(*)         REF
      CHARACTER*(*)         CORR

      INTEGER               TARGET
      INTEGER               OBS

      DOUBLE PRECISION      RADSPD


C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     TARGET    I   Body being observed.
C     ET        I   Ephemeris time of observation.
C     REF       I   Reference frame for observation.
C     CORR      I   Correction for observation.
C     OBS       I   Body from which observation is made.
C
C$ Detailed_Input
C
C     TARGET    NAIF ID of the body for which the radial speed is
C               needed. This body will normally be a vehicle or moon.
C
C     ET        Ephemeris time of observation in seconds past J2000.
C
C     REF       Reference frame for observation.  Normally this will
C               be J2000.
C
C     CORR      Correction on the state vector for the observation.
C               The possible choices are 'NONE', 'LT' 'LT+S'. 'LT is the
C               nominal choice
C
C     OBS       Body from which observation of TARGET is made.  This
C               should probably the primary of a moon or vehicle.
C
C$ Detailed_Output
C
C     RADSPD     The radial speed of TARGET w.r.t OBS at time ET with
C                correction CORR in reference frame REF.
C
C$ Parameters
C
C     None
C
C$ Files
C
C     The appropriate spk kernels should have been loaded prior to
C     calling this routine.
C
C$ Exceptions
C
C     None known.
C
C$ Particulars
C
C     This routine is nominally to be used by EXTAPP for determining
C     orbit extremum, but the use is not restricted to that operation.
C
C$ Examples
C
C     Return the radial speed of the moon, 301, wrt the earth, 399,
C     at some epoch ET.
C
C     VEL = RADSPD ( 301, ET, 'J2000', 'LT', 399 )
C
C$ Restrictions
C
C     None.   No error checking performed, zero, zilch, nada.
C
C$ Author_and_Institution
C
C     Ed Wright, JPL
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-    SPICELIB Version 1.5.0, 11-MAR-1997 (EDW)
C
C     Corrected documentation error in which speed, a scalar, was
C     referred to as velocity, a 3 vector.  Renamed the function call
C     RADSPD from RADVEL.
C
C-    SPICELIB Version 1.0.0, 11-MAR-1996 (EDW)
C
C-&


C$ Index_Entries
C
C     radial speed
C
C-&
C


C
C     SPICELIB functions.
C
      DOUBLE PRECISION      VDOT
      LOGICAL               RETURN


C
C     Locals.
C
      DOUBLE PRECISION      ET
      DOUBLE PRECISION      LT
      DOUBLE PRECISION      STATE( 6 )
      DOUBLE PRECISION      RHAT ( 3 )


C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RADSPD = -1.D0
         RETURN

      ELSE
         CALL CHKIN ( 'RADSPD' )

      END IF



C
C     Determine the state vector between the observer and the target
C     using the supplied corrections and frame reference at epoch ET.
C     STATE returns a 6 vector in which entries 1-3 are the position
C     components and 4-6 are the corresponding velocity components.
C

      CALL SPKEZ ( TARGET, ET, REF, CORR, OBS, STATE, LT )


C
C     Construct a radial unit vector from the position vector data in
C     STATE.
C

      CALL VHAT  ( STATE, RHAT )

C
C     Project the velocity components onto the RHAT vector.  RADSPD will
C     return with the radial velocity.  Note that the origins for the
C     velocity vector and the position unit vector do not coincide.
C     This is not a concern for this application since the vectors are
C     isomorphic in R3.
C
C               _       ^
C     v     =   v   .   r
C      rad
C

      RADSPD = VDOT ( STATE(4),  RHAT )


      CALL CHKOUT ( 'RADSPD' )

      RETURN
      END
