C$Procedure SPKRSF ( Read SPK scratch file)

      SUBROUTINE SPKRSF ( SCUNIT, BOD, ETMIN, ETMAX, BODCEN,
     .                    ERROR )

C$ Abstract
C
C     Subroutine to read the SPK information scratch file created by
C     SPKWSF and return the earliest and latest times for BOD
C     data segments.
C
C$ Copyright
C
C     Copyright (1996), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     SPK
C
C$ Declarations


      INTEGER               SCUNIT
      INTEGER               BOD
      INTEGER               BODCEN

      DOUBLE PRECISION      ETMIN
      DOUBLE PRECISION      ETMAX

      LOGICAL               ERROR

C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     SCUNIT     I   Unit number of the scratch file
C     BOD        I   NAIF ID of the body of interest
C     ETMIN      O   Minimum ET time of all segments for body BOD in the
C                    scratch file.
C     ETMAX      O   Maximum ET time of all the segments for BOD in the
C                    scratch file.
C     BODCEN     O   Body center for BOD if all BOD data segments have
C                    the same body center.
C     ERROR      O   Logical flag to indicate body center
C
C$ Detailed_Input
C
C     SCUNIT         The I/O logical number assigned to the scratch file
C                    when it was created.
C
C     BOD            The NAIF integer ID number for the body of interest
C                    to the user.
C
C$ Detailed_Output
C
C     ETMIN          The minimum (earliest) ET time in the scratch file
C                    for the body.
C
C     ETMAX          The maximum (latest) ET time in the scratch file.
C
C     BODCEN         The NAIF ID for body center of BOD if and only if
C                    (IFF) all segments have the same body center.
C
C     ERROR          Logical flag to indicate whether or not the data
C                    was successfully retrieved.
C
C$ Parameters
C
C       None.
C
C$ Files
C
C      The SPK scratch file is expected to exist.  It is created by a
C      call to SPKWSF.
C
C$ Exceptions
C
C      1) BODCEN returns a value only if all segments have the same
C         body center which will set the logical flag ERROR to
C        .FALSE.  Otherwise ERROR will be set to .TRUE.
C
C$ Particulars
C
C      The routine SPKWSF must have been called prior to calling
C      this routine, but this routine may be called as often as
C      needed afterwards.
C
C      This version of the routine does not determine if the data
C      coverage is continuous from ETMIN to ETMAX.
C
C$ Examples
C
C      Open an SPK kernel and pass its handle to SPKWSF which will
C      write a scratch file containing the body center, body, min time
C      and max time for every segment in the SPK file.
C
C      CALL SPKLEF ( '/kernel/spk/pff.bsp', HNDLE )
C      CALL SPKWSF( HNDLE, SCUNIT)
C
C      Read the scratch file for data concerning body -27 (VO2) and
C      return the earliest and latest times for all body -27
C      segments and the body center.
C
C      BOD = -27
C      CALL SPKRSF ( SCUNIT, BOD, ETMIN, ETMAX, BODCEN, ERROR )
C
C$ Restrictions
C
C      1) The body of interest, BOD, must be listed in the scratch
C         file for the program to proceed.
C
C$ Author_and_Institution
C
C      Ed Wright JPL
C
C$ Literature_References
C
C      None.
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 6-Aug-1996 (EDW)
C
C-&

C$ Index_Entries
C
C     SPK scratch file
C
C-&
C

      CHARACTER*(255)       TEMPCH

      INTEGER               BODY
      INTEGER               CENTER
      INTEGER               LASTCN

      DOUBLE PRECISION      BEGTIM
      DOUBLE PRECISION      ENDTIM

      LOGICAL               EOF
      LOGICAL               FOUND
      LOGICAL               FIRST


      CALL CHKIN ( 'SPKRSF' )


C
C     Initialize the logical flags.
C
      FOUND  = .FALSE.
      FIRST  = .TRUE.
      EOF    = .FALSE.
      ERROR  = .FALSE.
      BODCEN = -1

C
C     Rewind/reset the pointer to the beginning of the file so a new
C     read may be started.
C
      REWIND ( UNIT = SCUNIT )


C
C     Read the scratch file until an end of file is detected.
C
      DO WHILE ( .NOT. EOF )

         CALL READLN ( SCUNIT, TEMPCH, EOF )

C
C        Check for an end of file.  If it's not there, map the data
C        values off the character string to appropriate variables.
C
         IF ( .NOT. EOF ) THEN

C
C           We are interested in a particular body, ignore all others.
C           Read the string form the scratch file, obtaining the body
C           ID, center ID, segment begin and end times.
C
            READ( TEMPCH, * ) BODY, CENTER, BEGTIM, ENDTIM

            IF ( FIRST. AND. BODY. EQ. BOD ) THEN

C
C              This is the first incidence where the correct body ID was
C              read.  Initialize the working variables and flag a
C              successful find.
C
               ETMIN  = BEGTIM
               ETMAX  = ENDTIM
               FIRST  = .FALSE.
               LASTCN = CENTER
               BODCEN = CENTER
               FOUND  = .TRUE.

            ELSE IF ( BODY .EQ. BOD. AND. .NOT. FIRST ) THEN

C
C              Determine the min and max times, ET J2000, from the
C              scratch file data.
C
               IF ( BEGTIM .LT. ETMIN ) THEN
                  ETMIN = BEGTIM
               END IF


               IF ( ENDTIM .GT. ETMAX ) THEN
                  ETMAX = ENDTIM
               END IF

               IF ( LASTCN. NE. CENTER ) THEN
                  ERROR = .TRUE.
               END IF


            END IF


         END IF


      END DO


C
C     Make sure the body of interest was in the scratch file.  If not,
C     alert the user and bug out.
C
      IF ( .NOT. FOUND ) THEN
         CALL SETMSG ( 'Body # not found in SPKWSF scratch file.' )
         CALL ERRINT ( '#', BOD )
         CALL SIGERR ( 'BODY NOT FOUND' )
         STOP
      END IF


      CALL CHKOUT ( 'SPKRSF' )

      RETURN
      END
