#!/usr/local/bin/perl
#
#  This script generates PDS volume index file using infomation from the 
#  existing labels for a set of kernel files. List of kernels file names 
#  must be in the same format as input to the ``label_them_all.pl'' script
#  which was used to generate labels.
#
#  This script uses only START_TIME, STOP_TIME, PRODUCT_ID, DATA_SET_ID and
#  PRODUCT_CREATION_TIME from the labels.
#
#  Well ... this is about as much description as I have time for at the 
#  moment ... hopefully there is more to come ...
#
#  2.0.0, July 24, 2000, BVS/NAIF 
#
#     Changed index table format to make quotes adjacent to commas and 
#     label format to compte correct start character position and lenghts.
#
#  1.3.0, April 21, 2000, BVS/NAIF 
#
#     Relaxed DATA_SET_ID value check to allow more than one s/c ID
#     (separated by /).
#
#  1.2.0, January 10, 2000, BVS/NAIF 
#
#     Updated to generate index records for labels along with records for 
#     the data files, i.e. each data file will now have two entries --
#     one for the file itslef and another for the file label.
#
#  1.1.0, January 4, 2000, BVS/NAIF
#
#     Changed check filter for DATA_SET_ID. Corrected total computed index 
#     record length (by adding 1.) Uppercased file names and extensions.
#
#  1.0.0, April 26, 1999, BVS/NAIF
#
$scriptname = "make_index.pl";
$version    = "2.0.0, July 24, 2000, BVS/NAIF";
$indexfile  = "INDEX.TAB";
$indexlabel = "INDEX.LBL";

#
#  Display version.
#
print "\nScript that creates an index table file (and a label file for ".
      "it)\nfor a collection of SPICE kernels listed in a text file.\n\n".
      "Version $version.\n";

#
#  Get from the command line the name of the file containing list
#  of labeled kernels and other meta-keywords.
#
if ( @ARGV == 1 ) {
   $filelist = shift @ARGV;
   if ( ! -e $filelist ) {
      die "\nERROR:$scriptname: the file '$filelist' doesn't exist.\n\n";
   }
} else {
   die "\nUsage: > $scriptname <kernel_list_file_name>\n\n";
}

#
#  Check whether index.tab and index.LBL already exist.
#
if ( -e $indexfile ) {
   die "\nERROR:$scriptname: index table file '$indexfile' already exists.\n\n";
}
if ( -e $indexlabel ) {
   die "\nERROR:$scriptname: index label file '$indexlabel' already exists.\n\n";
}

#
#  Load list of kernel file(s).
#
print "\nLoading file names and required meta-information ...\n";
open( LIST, "< $filelist" ) || 
   die "\nERROR:$scriptname: cannot open '$filelist'.\n\n";
  
while ( $line = <LIST> ) {

   chop( $line );

   if     ( $line =~ /^FILE\s*=\s*/ ) {
   
      #
      #  Current line contains file name. Extract it and check whether the 
      #  exists.
      #
      $kernelfile = $line;
      $kernelfile =~ s/^FILE\s*=\s*//;
      
      if ( -e $kernelfile ) {
      
         #
         #  It does. Save it in the buffer, figure out label file name 
         #  and check whether the label file exists.
         #
         push( @kernels, $kernelfile );

         $labelfile  = $kernelfile;
         $labelfile  =~ s/\.\w+$/\.LBL/;
         
         if ( -e $labelfile ) {

            #
            #  It does too. Then figure out what kernel file extension is.
            #
            $kernelname = $kernelfile;
            $kernelname  =~ s/\.\w+$//;
            $kernelextension = $kernelfile;
            $kernelextension =~ s/$kernelname/\*/;
            $kernelextension =~ tr/a-z/A-Z/;
         
            #
            #  Save label name and file extension in the buffers.
            #
            $labels{$kernelfile} = $labelfile;
            $extensions{$kernelextension} = "";
            
         }  else {
            print "   ERROR: '$labelfile' doesn't exist.\n";
         }
         
      } else {
         print "   ERROR: '$kernelfile' doesn't exist.\n";
      }

   } elsif ( $line =~ /^VOLUMEID\s*=\s*/ ) {     

      $VOLUMEID = $line;
      $VOLUMEID =~ s/^VOLUMEID\s*=\s*//;
         
   } else {

      #
      #  Well ... we do nothing with other lines ... at least in this version.
      #
   }

}

close( LIST ) || 
   die "\nERROR:$scriptname: cannot close '$filelist'.\n\n"; 
print "done.\n";

#
#  Do consistency check on collected information.
#
print "\nChecking loaded file names ...\n";

if ( @kernels == 0 ) {
   die "\nERROR:$scriptname: didn't collect any kernel file names provided ".
       "using FILE = ... keyword/value assignments in the config file ".
       "'$filelist'.\n\n";
}

@tmparr = values %labels;
if ( $#kernels != $#tmparr ) {
   print  "\nERROR:$scriptname: number of collected kernel file ".
          "names ($#kernels+1) doesn't match number of label file ".
          "names ($#tmparr+1).\n" ;
   print "\nThe following kernel/label names were collected:\n\n";
   foreach $file ( sort @kernels ) {
      printf "   %-37s   %-37s\n", $file, $labels{$file};
   }
   die "\n";
}

if ( ! ( $VOLUMEID =~ /\s*[0-9A-Z]+_\d+\s*/ ) ) {
   die "\nERROR:$scriptname: volume ID was not provided in the config ".
       "file '$filelist' using VOLUMEID = ... keyword/value assignment ".
       "or it didn't look like CCCC_DDDD, where C is an appercase ".
       "character and D is a digit.\n\n";
}

print "done.\n";

print "\nCollected $#kernels+1 file/label names. Collected the ".
      "following set of file\nextensions:\n";
@tmparr = keys %extensions;
foreach $file ( @tmparr ) {
   print "   $file\n";
}

#
#  Loop through label files and pull out necessary information 
#  from them.
#
print "\nCollecting index table values from labels ...\n";

$STARTTIMEL = "0";
$STOPTIMEL = "0";
$FILENAMEL = "0";
$DATASETIDL = "0";
$CREATIONTIMEL = "0";

foreach $kernelfile ( @kernels ) {

   open( LABEL, "< $labels{$kernelfile}" ) || 
      die "\nERROR:$scriptname: cannot open label file ".
          "'$labels{$kernelfile}'.\n\n";
  
   while ( $line = <LABEL> ) {
   
      chop( $line );
      
      if      ( $line =~ /^START_TIME\s*=\s*/ ) {
      
         $STARTTIME{$kernelfile} = $line;
         $STARTTIME{$kernelfile} =~ s/^START_TIME\s*=\s*//;
         if ( length($STARTTIME{$kernelfile}) > $STARTTIMEL ) {
            $STARTTIMEL = length($STARTTIME{$kernelfile});
         }
         
      } elsif ( $line =~ /^STOP_TIME\s*=\s*/ ) {
      
         $STOPTIME{$kernelfile} = $line;
         $STOPTIME{$kernelfile} =~ s/^STOP_TIME\s*=\s*//;
         if ( length($STOPTIME{$kernelfile}) > $STOPTIMEL ) {
            $STOPTIMEL = length($STOPTIME{$kernelfile});
         }
         
      } elsif ( $line =~ /^PRODUCT_ID\s*=\s*/ ) {
      
         $FILENAME{$kernelfile} = $line;
         $FILENAME{$kernelfile} =~ s/^PRODUCT_ID\s*=\s*//;
         $FILENAME{$kernelfile} =~ s/\"//g;
         
      } elsif ( $line =~ /^DATA_SET_ID\s*=\s*/ ) {
      
         $DATASETID{$kernelfile} = $line;
         $DATASETID{$kernelfile} =~ s/^DATA_SET_ID\s*=\s*//;
         $DATASETID{$kernelfile} =~ s/\"//g;
         if ( length($DATASETID{$kernelfile}) > $DATASETIDL ) {
            $DATASETIDL = length($DATASETID{$kernelfile});
         }
         
      } elsif ( $line =~ /^PRODUCT_CREATION_TIME\s*=\s*/ ) {
      
         $CREATIONTIME{$kernelfile} = $line;
         $CREATIONTIME{$kernelfile} =~ s/^PRODUCT_CREATION_TIME\s*=\s*//;
         if ( length($CREATIONTIME{$kernelfile}) > $CREATIONTIMEL ) {
            $CREATIONTIMEL = length($CREATIONTIME{$kernelfile});
         }
         
      } else {
      
         #
         #  We don't use other label keyword in indexes.
         #
         
      }     
      
   }
   
   close( LABEL ) || 
      die "\nERROR:$scriptname: cannot close label file  ".
          "'$labels{$kernelfile}'.\n\n";
          
   #
   #  Check all values that we have collected from the current label.
   #
   if ( ! ( $STARTTIME{$kernelfile} =~ 
      /^\d{4,4}-\d{2,2}-\d{2,2}T\d{2,2}:\d{2,2}:\d{2,2}/ ) && 
        ! ( $STARTTIME{$kernelfile} =~ /\"N\/A\"/ ) ) {
      die "\nERROR:$scriptname: START_TIME '$STARTTIME{$kernelfile}' ".
          "from the label '$labels{$kernelfile}' doesn't match expected ".
          "ISO format YYYY-MM-DDTHR:MN:SC or it wasn't present in ".
          "the label at all.\n\n";
   }
   
   if ( ! ( $STOPTIME{$kernelfile} =~ 
      /^\d{4,4}-\d{2,2}-\d{2,2}T\d{2,2}:\d{2,2}:\d{2,2}/ ) && 
        ! ( $STOPTIME{$kernelfile} =~ /\"N\/A\"/ ) ) {
      die "\nERROR:$scriptname: STOP_TIME '$STOPTIME{$kernelfile}' ".
          "from the label '$labels{$kernelfile}' doesn't match expected ".
          "ISO format YYYY-MM-DDTHR:MN:SC or it wasn't present in ".
          "the label at all.\n\n";
   }
   
   $tmpvar = $FILENAME{$kernelfile};
   if ( ! ( $kernelfile =~ /$tmpvar/ ) ) {
      die "\nERROR:$scriptname: PRODUCT_ID '$tmpvar' ".
          "from the label '$labels{$kernelfile}' doesn't contain   ".
          "actual kernel file name.\n\n";
   } else {
      $FILENAME{$kernelfile} = $kernelfile;
      $FILENAME{$kernelfile} =~ tr/a-z/A-Z/;
      if ( length($FILENAME{$kernelfile}) > $FILENAMEL ) {
         $FILENAMEL = length($FILENAME{$kernelfile});
      }
      if ( ( length($labels{$kernelfile}) ) > $FILENAMEL ) {
         $FILENAMEL = length($labels{$kernelfile});
      }
   }
   
   $tmpvar = $DATASETID{$kernelfile};
   if ( ! ( $tmpvar =~ /^[0-9A-Z\/]+-[A-Z]+-SPICE-6-V\d\.\d$/ ) ) {
      die "\nERROR:$scriptname: DATA_SET_ID '$tmpvar' ".
          "from the label '$labels{$kernelfile}' doesn't match expected ".
          "pattern CCCCCC-C-SPICE-6-CCCC-VD.D.\n\n";
   }
   
   if ( ! ( $CREATIONTIME{$kernelfile} =~ 
      /^\d{4,4}-\d{2,2}-\d{2,2}T\d{2,2}:\d{2,2}:\d{2,2}/ ) ) {
      die "\nERROR:$scriptname: PRODUCT_CREATION_TIME ".
          "'$CREATIONTIME{$kernelfile}' ".
          "from the label '$labels{$kernelfile}' doesn't match expected ".
          "ISO format YYYY-MM-DDTHR:MN:SC.### or it wasn't present in ".
          "the label at all.\n\n";
   }

}
print "done.\n";

#
#  Write index table file.
#
print "\nWriting index table file ...\n";

open( TABLE, ">$indexfile" ) ||
   die "\nERROR:$scriptname: cannot open index table file '$indexfile'.\n\n";
   
foreach $kernelfile ( @kernels ) {
   #
   #  Print records for the data file.
   #
   printf TABLE "\%-${STARTTIMEL}s,\%-${STOPTIMEL}s,\"\%-${FILENAMEL}s\",".
          "\"\%-${DATASETIDL}s\",\%${CREATIONTIMEL}s\n", 
          $STARTTIME{$kernelfile}, $STOPTIME{$kernelfile}, 
          $FILENAME{$kernelfile}, $DATASETID{$kernelfile}, 
          $CREATIONTIME{$kernelfile};      
   #
   #  Write almost identical record for the label.
   #
   $labelfile = $labels{$kernelfile};
   printf TABLE "\%-${STARTTIMEL}s,\%-${STOPTIMEL}s,\"\%-${FILENAMEL}s\",".
          "\"\%-${DATASETIDL}s\",\%${CREATIONTIMEL}s\n", 
          $STARTTIME{$kernelfile}, $STOPTIME{$kernelfile}, 
          $labelfile, $DATASETID{$kernelfile}, 
          $CREATIONTIME{$kernelfile};      
}

close( TABLE ) ||
   die "\nERROR:$scriptname: cannot close index table file '$indexfile'.\n\n";
   
print "done.\n";

#
#  Values that will be substituted in the index table label text:
#
$VOLUMEID         =  $VOLUMEID;

$RECORDBYTES      =  $STARTTIMEL + 
                     $STOPTIMEL + 
                     $FILENAMEL + 
                     $DATASETIDL + 
                     $CREATIONTIMEL + 10;

$FILERECORDS      = 2 * ( $#kernels + 1 );

$ROWBYTES         = $RECORDBYTES;

$ROWS             = $FILERECORDS;

$COLUMNS          = "5";

$FILEEXTENSIONS   = "\n";
@tmparr           = keys %extensions;
foreach $file ( @tmparr ) {
  $FILEEXTENSIONS = $FILEEXTENSIONS.
                    "                               \"$file\",\n";
}
chop( $FILEEXTENSIONS );
chop( $FILEEXTENSIONS );
$FILEEXTENSIONS= $FILEEXTENSIONS."\n";

@STARTPOS         =( 1, 
                     $STARTTIMEL + 2,
                     $STARTTIMEL + $STOPTIMEL + 4,
                     $STARTTIMEL + $STOPTIMEL + $FILENAMEL + 7,
                     $STARTTIMEL + $STOPTIMEL + $FILENAMEL + $DATASETIDL + 9 );
              
@FIELDLENGTH      =( $STARTTIMEL,
                     $STOPTIMEL, 
                     $FILENAMEL, 
                     $DATASETIDL, 
                     $CREATIONTIMEL );

#
#  Write table label file.
#
print "\nWriting index label file ...\n";

open( TABLELABEL, ">$indexlabel" ) ||
   die "\nERROR:$scriptname: cannot open index label file '$indexlabel'.\n\n";

print TABLELABEL <<"EndOfLabelText";
PDS_VERSION_ID             = PDS3 
VOLUME_ID                  = $VOLUMEID
RECORD_TYPE                = FIXED_LENGTH
RECORD_BYTES               = $RECORDBYTES
FILE_RECORDS               = $FILERECORDS 
^INDEX_TABLE               = "INDEX.TAB"
              
OBJECT                     = INDEX_TABLE

  INTERCHANGE_FORMAT       = ASCII
  ROW_BYTES                = $ROWBYTES
  ROWS                     = $ROWS
  COLUMNS                  = $COLUMNS
  INDEX_TYPE               = SINGLE
  INDEXED_FILE_NAME        = { $FILEEXTENSIONS                             }

  OBJECT                   = COLUMN
    NAME                   = START_TIME
    DATA_TYPE              = "TIME"
    START_BYTE             = $STARTPOS[0]
    BYTES                  = $FIELDLENGTH[0]
    DESCRIPTION            = "Start time of the product."
  END_OBJECT               = COLUMN
             
  OBJECT                   = COLUMN
    NAME                   = STOP_TIME
    DATA_TYPE              = "TIME"
    START_BYTE             = $STARTPOS[1]
    BYTES                  = $FIELDLENGTH[1]
    DESCRIPTION            = "Stop time of the product."
  END_OBJECT               = COLUMN

  OBJECT                   = COLUMN
    NAME                   = FILE_SPECIFICATION_NAME
    DATA_TYPE              = "CHARACTER"
    START_BYTE             = $STARTPOS[2]
    BYTES                  = $FIELDLENGTH[2]
    DESCRIPTION            = "File name."
  END_OBJECT               = COLUMN

  OBJECT                   = COLUMN
    NAME                   = DATA_SET_ID
    DATA_TYPE              = "CHARACTER"
    START_BYTE             = $STARTPOS[3]
    BYTES                  = $FIELDLENGTH[3]
    DESCRIPTION            = "Data set ID."
  END_OBJECT               = COLUMN

  OBJECT                   = COLUMN
    NAME                   = PRODUCT_CREATION_TIME
    DATA_TYPE              = "TIME"
    START_BYTE             = $STARTPOS[4]
    BYTES                  = $FIELDLENGTH[4]
    DESCRIPTION            = "Product creation time."
  END_OBJECT               = COLUMN

END_OBJECT                 = INDEX_TABLE
END
EndOfLabelText

close( TABLELABEL ) ||
   die "\nERROR:$scriptname: cannot close index label file '$indexlabel'.\n\n";
print "done.\n";

print "\nIndex table file '$indexfile' and index label file ".
      "'$indexlabel' were\nsuccessfuly created\n\n";

#
#  All done.
#
exit;

