#!/usr/bin/perl
#
# Convert ESOC's event file (EVTF*.MEX) to prediCkt orientation schedule
# assuming Earth-pointed orientation for all time when we New Norcia
# is visible (bound by NNO_AOS_10/NNO_LOS_10 events) and nadir pointed for
# all other times.
#
# Takes EVTF lines from STDIN, dump prediCkt schedule lines to STDOUT.
#
# THE SCRIPT DOES NOT DO ANY ERROR CHECKING AND WILL BLOW UP IF INPUT
# IS NOT EVTF* FILE!!!
#
# Version 1.1.0 -- November 10, 2003 -- BVS/NAIF
#
#   Changed CK ID from -41001 to -41001 to be consistent with the
#   definition section specifying directions/orientations for
#   MEX_SC_REF frame.
#
# Version 1.0.0 -- April 8, 2003 -- BVS/NAIF
#

#
# Set LSK location and slew duration in seconds.
#
$lsk  = "/kernels/gen/lsk/naif0007.tls";
$slew = "300";

#
# Get the first line.
#
$line = <STDIN>;

#
# Pull the time from it. Convert UTC to calendar ET and buffer it.
# Set "first" flag.
#
@tokens = split( ' ', $line );
$utc = $tokens[3];
chop( $utc );
$utc = "20".$utc;
$etsec = `chronos -setup $lsk -from utc -to et -totype seconds -time $utc -nolabel 2>&1`;
chop( $etsec );
$etcal = `chronos -setup $lsk -from et -fromtype seconds -to et -format 'YYYY-MM-DD-HR:MN:SC' -time $etsec -nolabel 2>&1`;
chop( $etcal );
$etbeg = $etcal;

$first = "TRUE";

#
# Print schedule header.
#
print <<EndBlock;

      \\begindata

          CK-SCLK               =  41
          CK-SPK                = -41
          CK-FRAMES             = -41001
EndBlock

#
# Get lines one by one.
#
while ( $line = <STDIN> ) {

  if ( $line =~ /NNO_AOS_10/ || $line =~ /NNO_LOS_10/ ) {

    #
    # Pull the time from it. Convert UTC to ET seconds.
    #
    @tokens = split( ' ', $line );
    $utc = $tokens[3];
    chop( $utc );
    $utc = "20".$utc;
    $etsec = `chronos -setup $lsk -from utc -to et -totype seconds -time $utc -nolabel 2>&1`;
    chop( $etsec );

    #
    # Check for start of Earth point.
    #
    if ( $line =~ /NNO_AOS_10/ ) {

      #
      # Convert ET seconds to calendar ET --> Earth-point start time.
      #
      $etcal = `chronos -setup $lsk -from et -fromtype seconds -to et -format 'YYYY-MM-DD-HR:MN:SC' -time $etsec -nolabel 2>&1`;
      chop( $etcal );
      $earthbeg = $etcal;

      #
      # Convert ET seconds - slew to calendar ET --> nadir end time.
      #
      $etsec = $etsec - $slew;
      $etcal = `chronos -setup $lsk -from et -fromtype seconds -to et -format 'YYYY-MM-DD-HR:MN:SC' -time $etsec -nolabel 2>&1`;
      chop( $etcal );
      $nadirend = $etcal;

      #
      # If first is true, s/c was nadir pointed at the start of the file.
      #
      if ( $first eq "TRUE" ) {
        print <<EndBlock;

          CK-41001ORIENTATION  += 'NadirPointed'
          CK-41001START        += \@$etbeg
EndBlock
        $first = "FALSE";
      }

      #
      # Finish nadir window, start Earth-pointed window.
      #
      print <<EndBlock;
          CK-41001STOP         += \@$nadirend

          CK-41001ORIENTATION  += 'EarthPointed'
          CK-41001START        += \@$earthbeg
EndBlock

    }

    #
    # For NNO_LOS_10 compute ET and ET+5min.
    #
    if ( $line =~ /NNO_LOS_10/ ) {

      #
      # Convert ET seconds to calendar ET --> Earth-point stop time.
      #
      $etcal = `chronos -setup $lsk -from et -fromtype seconds -to et -format 'YYYY-MM-DD-HR:MN:SC' -time $etsec -nolabel 2>&1`;
      chop( $etcal );
      $earthend = $etcal;

      #
      # Convert ET seconds + slew to calendar ET --> nadir start time.
      #
      $etsec = $etsec + $slew;
      $etcal = `chronos -setup $lsk -from et -fromtype seconds -to et -format 'YYYY-MM-DD-HR:MN:SC' -time $etsec -nolabel 2>&1`;
      chop( $etcal );
      $nadirbeg = $etcal;

      #
      # If first is true, s/c was Earth-pointed at the start of the file.
      #
      if ( $first eq "TRUE" ) {
         print <<EndBlock;

          CK-41001ORIENTATION  += 'EarthPointed'
          CK-41001START        += \@$etbeg
EndBlock
         $first = "FALSE";
      }

      #
      # Finish Earth-pointed window, start nadir window.
      #
      print <<EndBlock;
          CK-41001STOP         += \@$earthend

          CK-41001ORIENTATION  += 'NadirPointed'
          CK-41001START        += \@$nadirbeg
EndBlock

    }

  }

  $lastline = $line;

}

#
# Pull out last time.
#
@tokens = split( ' ', $lastline );
$utc = $tokens[3];
chop( $utc );
$utc = "20".$utc;
$etsec = `chronos -setup $lsk -from utc -to et -totype seconds -time $utc -nolabel 2>&1`;
chop( $etsec );
$etcal = `chronos -setup $lsk -from et -fromtype seconds -to et -format 'YYYY-MM-DD-HR:MN:SC' -time $etsec -nolabel 2>&1`;
chop( $etcal );
$etend = $etcal;

#
# Print schedule footer.
#
print <<EndBlock;
          CK-41001STOP         += \@$etend

      \\begintext
EndBlock

#
# all done.
#
exit(0);

